export class FileService {
    constructor(){
        this.userService = null;

        this.apiServer = [];
        
        this.product = "AC";
        this.fileList = [];
    }

    init(opts){
        this.userService = opts.userService;

        var link = this.userService.getService("fileService");
        if(link==null || link.length<5){
            return false;
        }
        this.apiServer["file"] = link + "/object";
        this.apiServer["entity"] = link + "/entity";
        this.apiServer["regist"] = link + "/regist";
        // this.apiServer["file"] = "http://localhost:8090/api/file/object";
        // this.apiServer["entity"] = "http://localhost:8090/api/file/entity";
        // this.apiServer["regist"] = "http://localhost:8090/api/file/regist";

        return true;
    }

    getProduct(){
        return this.product;
    }
    getFileLink(fid){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["file"] + "/download/" + fid + "/" + rand;
        return link;
    }
    getTempLink(uuid){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["file"] + "/temp/" + uuid + "/" + rand;
        return link;
    }
    getFilePath(fid, udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["file"] + "/path/" + fid + "/" + rand;
        this.execute("GET", link, "path", fid, udata, callback, myapp);
        return link;
    }

    uploadFile(file, udata, callback, myapp){
        if(this.fileList.length==0){
            this.doFileUpload(file, udata, callback, myapp);
        } else {
            var myobj = {file:file, udata:udata, callback:callback, myapp:myapp};
            this.fileList.push(myobj);
        }
    }
    doFileUpload(file, udata, callback, myapp){
        var token = this.userService.getToken();
        if(token==null || token.length<12){
            callback("upload", file, "user token not exists", -1, udata, myapp);
            return;
        }

        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["file"] + "/upload/" + this.product + "/" + rand;

        var xhr = new XMLHttpRequest();
        xhr.open("POST", link, true);
        xhr.setRequestHeader("token", token);

        var data = new FormData();
        data.append("state", file.state);
        data.append("flags", file.flags);
        if(file.fid && file.fid.length>0){
            data.append("fid", file.fid);
        }
        data.append("product", this.product);
        data.append("file", file.fileobj);

        var myobj = this;
        xhr.upload.onprogress=function(evt){
            var progress = (evt.loaded / evt.total * 100).toFixed(0);
            callback("progress", file, null, progress, udata, myapp);
        },
        xhr.onreadystatechange=function(){
            if(xhr.readyState==4){
                if(xhr.status>=400){
                    callback("upload", file, xhr.statusText, null, udata, myapp);
                } else if(xhr.status==200){
                    console.log(xhr.responseText);
                    var rdata = JSON.parse(xhr.responseText);
                    if(rdata.status==200 && rdata.result.count==1){
                        var fobj = rdata.result.docs[0];
                        callback("upload", file, null, fobj, udata, myapp);
                    } else if(rdata.status!=200){
                        callback("upload", file, rdata.error, null, udata, myapp);
                    }
                }
                if(myobj.fileList.length>0){
                    var curr = myobj.fileList[0];
                    myobj.fileList.splice(0, 1);

                    myobj.doFileUpload(curr.file, curr.udata, curr.callback, curr.myapp);
                }
            }
        },
     
        xhr.send(data);
    }

    setState(fid, state, udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["file"] + "/state/" + fid + "/" + state + "/" + rand;
        
        var params = {fid:fid, state:state};
        this.execute("GET", link, "state", params, udata, callback, myapp);
    }
    setFlags(fid, flags, udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["file"] + "/flags/" + fid + "/" + flags + "/" + rand;

        var params = {fid:fid, flags:flags};
        this.execute("GET", link, "flags", params, udata, callback, myapp);
    }
    rename(fid, name, oname, state, udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["file"] + "/rename/" + fid + "/" + rand;

        var params = {fid:fid, name:name, oname:oname};
        this.execute("POST", link, "rename", params, udata, callback, myapp);
    }
    getFileInfo(fid, udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["file"] + "/get/" + fid + "/" + rand;
        this.execute("GET", link, "get", fid, udata, callback, myapp);
    }
    deleteFile(fid, udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["file"] + "/delete/" + fid + "/" + rand;
        this.execute("GET", link, "delete", fid, udata, callback, myapp);
    }
    removeFiles(fids, udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["file"] + "/remove/" + rand;

        var params = {fids:fids};
        this.execute("POST", link, "remove", params, udata, callback, myapp);
    }
    
    doFileCall(action, params, udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["file"] + "/" + action + "/" + rand;
        this.execute("POST", link, action, params, udata, callback, myapp);
    }
    doFileEntityCall(action, params, udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["entity"] + "/" + action + "/" + rand;
        this.execute("POST", link, action, params, udata, callback, myapp);
    }
    execute(method, link, action, params, udata, callback, myapp){
        var xhr = new XMLHttpRequest();
        xhr.timeout=60000;
        xhr.open(method, link, true);
        
        var token = this.userService.getToken();
        if(token==null || token.length==0){
            callback(action, params, "user token not exists", null, udata, myapp);
            return;
        }
        xhr.setRequestHeader("token", token);

        var myobj = this;
        xhr.onreadystatechange=function(){
            if(xhr.readyState==4){
                myobj.onFileCall(action, params, xhr, udata, callback, myapp);
            }
        }

        var text = null;
        if(params && method=='POST'){
            text = JSON.stringify(params);
        }
        xhr.send(text);
    }
    onFileCall(action, params, xhr, udata, callback, myapp){
        if(xhr.status!=200){
            callback(action, params, xhr.statusText, null, udata, myapp);
            return;
        }

        var rdata = JSON.parse(xhr.responseText);
        if(rdata.status==200){
            callback(action, params, null, rdata.result, udata, myapp);
        } else {
            if(rdata.error && rdata.error.length>0){
                callback(action, params, rdata.error, rdata.result, udata, myapp);
            } else {
                callback(action, params, "request failed", rdata.result, udata, myapp);
            }
        }
    }
}

var fileService = null;
export function provideFileService(opts){
    if(fileService==null){
        var temp = new FileService();
        if(!temp.init(opts)){
            return null;
        }
        fileService = temp;
    }
    return fileService;
}