
export class UserService {
    constructor(){
        this.token = "";
        this.userInfo = null;

        this.debug = true;
        this.apiServer = [];
        if(this.debug){
            this.apiServer["user"] = "http://172.16.0.153:8094/api/user/object";
            this.apiServer["team"] = "http://172.16.0.153:8094/api/team/member";
        } else {
            this.apiServer["user"] = "/api/user/object";
            this.apiServer["team"] = "/api/team/member";
        }

        this.userNames = [];
        this.mapUserNames = [];
        this.userRights = {
            logined:false,
            name:'',
            current:{},

            role:'email',
            roles:['creator', 'agent', 'check', 'submit', 'ofile', 'bill', 'collision', 'cert', 'renewal', 'email', 'manager', 'sysadmin'],
            acps:[],
        };
    }
    init(opts){
        var text = window.localStorage.getItem("ntdtm.usertoken");
        if(text){
            this.token = text;
        }

        var i=0, temp=null, myarr=null;
        text = window.localStorage.getItem("ntdtm.userinfo");
        if(text){
            this.userInfo = JSON.parse(text);
        }

        text = window.localStorage.getItem("ntdtm.userrights");
        if(text){
            this.userRights = JSON.parse(text);
        }

        text = window.localStorage.getItem("ntdtm.usernames");
        if(text){
            myarr = JSON.parse(text);
            if(myarr && myarr.length>0){
                for(i=0; i<myarr.length; i++){
                    temp = myarr[i];
                    this.userNames.push(temp);
                    this.mapUserNames[temp.uid] = temp;
                }
            }
        }

        text = localStorage.getItem("ntdtm.accessPoints");
        if(text){
            myarr = JSON.parse(text);
            if(myarr){
                this.accessPoints = myarr;
            }
        }
        return true;
    }
    getWindowTitle(workbench, bench, panel){
        var myobj = this.getAccessPoint(workbench, bench, panel);
        if(myobj!=null){
            return myobj.name + "-NTDTM";
        }
        return "永新商标-NTDTM";
    }
    getToken(){
        return this.token;
    }
    getUID(){
        if(this.userInfo){
            return this.userInfo.uid;
        }
        return null;
    }
    getUserRole(){
        if(this.userInfo){
            return this.userInfo.role;
        }
        return null;
    }
    isRoleOf(role){
        //20221223  卢振华 userRights下没有uid属性，且userrights本身不是数组没有length属性
        if(!(this.userRights && this.userRights.roles.length>0)){
            return false;
        }

        var myarr = this.userRights.roles;
        for(var i=0; i<myarr.length; i++){
            if(myarr[i]==role){
                return true;
            }
        }
        return false;
    }
    getUserName(uid){
        if (uid==null || uid==undefined || uid.length<6) {
            return "";
        }
        var item = this.mapUserNames[uid];
        if(item!=null){
            return item.ticn;
        }
        this.getName(uid);
        item = this.mapUserNames[uid];
        if(item!=null){
            return item.ticn;
        }
        return "";
    }
    getUserInfo(){
        return this.userInfo;
    }
    getUserRights(){
        return this.userRights;
    }
    getService(name){
        var myarr = (this.userInfo ? this.userInfo.settings : null);
        if(myarr && myarr.length>0){
            for(var i=0; i<myarr.length; i++){
                var temp = myarr[i];
                if(temp.name==name){
                    if(this.debug){
                        return temp.value;
                    }

                    //2022-08-01 10:21 lhp, 去掉绝对地址，利用nginx反向代理转发请求，方便网站直接向外网开放
                    var link = temp.value;
                    var pos = link.indexOf("/api/");
                    if(pos>0){
                        link = link.substr(pos);
                    } else {
                        pos = link.indexOf("/template");
                        if(pos>0){
                            link = link.substr(pos);
                        }
                    }
                    console.log("getService", name, link);
                    return link;
                }
            }
        }
        return null;
    }
    getName(uid){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["user"] + "/name/" + rand;

        var params = {uid:uid};
        this.execute("POST", false, link, "name", params, null, null, null);
    }
    isOnline(){
        if(this.token==null || this.token==""){
            return false;
        }

        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["user"] + "/online/" + rand;

        this.execute("GET", false, link, "online", null, null, null, null);
        if(this.token && this.token.length>6){
            return true;
        }
        return false;
    }
    logout(udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["user"] + "/logout/" + rand;

        this.execute("GET", true, link, "logout", null, udata, callback, myapp);
    }
    login(params, udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["user"] + "/login/" + rand;
        this.execute("POST", true, link, "login", params, udata, callback, myapp);
    }
    changePass(params, udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["user"] + "/changePass/" + rand;
        this.execute("POST", true, link, "changePass", params, udata, callback, myapp);
    }
    recommendUserList(params, udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["user"] + "/recommend/" + rand;
        this.execute("POST", true, link, "recommend", params, udata, callback, myapp);
    }
    getTeamMembers(params, udata, callback, myapp){
        var rand = Math.round(Math.random() * 100000);
        var link = this.apiServer["team"] + "/list/" + rand;
        this.execute("POST", true, link, "team", params, udata, callback, myapp);
    }
    execute(method, async, link, action, params, udata, callback, myapp){
        var xhr = new XMLHttpRequest();
        if(async){
            xhr.timeout = 15000;
        }
        xhr.open(method, link, async);
        
        if(action!='login'){
            var token = this.getToken();
            if(token==null || token.length==0){
                if(callback!=null){
                    callback(action, params, "user token not exists", null, udata, myapp);
                }
                return;
            }
            xhr.setRequestHeader("token", token);
        }

        var myobj = this;
        xhr.onreadystatechange=function(){
            if(xhr.readyState==4){
                myobj.onCallDone(action, params, xhr, udata, callback, myapp);
            }
        }

        var text = null;
        if(params && method=='POST'){
            text = JSON.stringify(params);
        }
        xhr.send(text);
    }
    onCallDone(action, params, xhr, udata, callback, myapp){
        if(xhr.status!=200){
            if(callback!=null){
                callback(action, params, xhr.statusText, null, udata, myapp);
            }
            return;
        }

        var rdata = JSON.parse(xhr.responseText);
        if(rdata.status==200){
            if(action=='login'){
                this.userInfo = rdata.result;
                this.token = this.userInfo.token;
                this.settings = this.userInfo.settings;
                this.initUserRights(rdata.result);

                var text = JSON.stringify(this.userInfo);
                window.localStorage.setItem("ntdtm.userinfo", text);
                window.localStorage.setItem("ntdtm.usertoken", this.token);
            } else if(action=='logout'){
                this.userRights = null;
                this.userInfo = null;
                this.token = "";
                window.localStorage.removeItem("ntdtm.userinfo");
                window.localStorage.removeItem("ntdtm.usertoken");
                window.localStorage.removeItem("ntdtm.userrights");
            } else if(action=='name'){
                if(rdata.result.count>0 && rdata.result.docs){
                    this.cacheUserNames(rdata.result.docs);
                }
            }

            if(callback!=null){
                callback(action, params, null, rdata.result, udata, myapp);
            }
        } else if(callback!=null){
            if(action=="online"){
                this.userInfo = null;
                this.token = "";
                window.localStorage.removeItem("ntdtm.userinfo");
                window.localStorage.removeItem("ntdtm.usertoken");
                window.localStorage.removeItem("ntdtm.userrights");
            }

            if(rdata.error && rdata.error.length>0){
                callback(action, params, rdata.error, rdata.result, udata, myapp);
            } else {
                callback(action, params, "request failed", rdata.result, udata, myapp);
            }
        }
    }
    cacheUserNames(myarr){
        for(var i=0; i<myarr.length; i++){
            var item = myarr[i];
            this.userNames.push(item);
            this.mapUserNames[item.uid] = item;
        }

        var text = JSON.stringify(this.userNames);
        window.localStorage.setItem("ntdtm.usernames", text);
    }
    initUserRights(param){
        this.userRights = {};
        this.userRights.state = "online";
        this.userRights.name = param.name;
        this.userRights.role = param.role;

        var i=0, temp = null, myarr=null, item = null, roles = [], acps = [], modules = [];
        if(param.roles && param.roles){
            myarr = param.roles;
            for(i=0; i<myarr.length; i++){
                temp = myarr[i];
                if(temp.prior==0){
                    this.userRights.role = temp.role;
                }
                roles.push(temp.role);

                if(temp.items && temp.items.length>0){
                    var tarr = temp.items;
                    for(var t=0; t<tarr.length; t++){
                        item = tarr[t];
                        if(this.findAccessPoints(item, false, acps)==null){
                            acps.push(item);
                            if(modules.indexOf(item.module)<0){
                                modules.push(item.module);
                            }
                        }
                    }
                }
            }
        } else {
            roles.push(param.role);
        }
        this.userRights.roles = roles;
        this.userRights.acps = acps;
        this.userRights.modules = modules;

        var workbench = [];
        for(i=0; i<roles.length; i++){
            temp = this.getWorkbench(roles[i]);
            if(temp){
                temp.role = roles[i];
                if(roles[i]==this.userRights.role){
                    this.userRights.current = temp;
                }
                workbench.push(temp);
            }
        }
        if(this.userRights.current==undefined || this.userRights.current==null){
            if(workbench.length>0){
                this.userRights.current = workbench[0];
            }
        }
        this.userRights.workbench = workbench;

        var text = JSON.stringify(this.userRights);
        localStorage.setItem("ntdtm.userrights", text);
    }
    findAccessPoints(item, remove, myarr){
        for(var i=0; i<myarr.length; i++){
            var temp = myarr[i];
            if(temp.path==item.path){
                if(remove){
                    myarr.splice(i, 1);
                }
                return temp;
            }
        }
        return null;
    }
    getWorkbench(role){
        var myobj = {};
        switch(role){
            case "agent":
            case "assist":
            case "clerk":
                myobj = {workbench:'agent', bench:"task", panel:'ontime'};
                break;
            case "creator":
                myobj = {workbench:'agent', bench:'case', panel:'volume'};
                break;
            case "check":
                myobj = {workbench:'flow', bench:'check', panel:'list'};
                break;
            case "submit":
                myobj = {workbench:'flow', bench:'check', panel:'submit'};
                break;
            case "cert":
                myobj = {workbench:'flow', bench:'cert', panel:'list'};
                break;
            case "ofile":
                myobj = {workbench:'flow', bench:'ofile', panel:'import'};
                break;
            case "renewal":
                myobj = {workbench:'flow', bench:'renewal', panel:'list'};
                break;
            case "collision":
                myobj = {workbench:'flow', bench:'conflict', panel:'list'};
                break;
            case "bill":
                myobj = {workbench:'account', bench:'bill', panel:'list'};
                break;
            case "instruct":
                myobj = {workbench:'instruct', bench:'inbox', panel:'list'};
                break;
            case "message":
                myobj = {workbench:'message', bench:'inbox', panel:'list'};
                break;
            case "kmo":
                myobj = {workbench:'km', bench:'flow', panel:'list'};
                break;
            case "risk":
                myobj = {workbench:'risk', bench:'conflict', panel:'list'};
                break;
            case "manager":
                myobj = {workbench:'collect', bench:'task', panel:'list'};
                break;
            case "template":
                myobj = {workbench:'admin', bench:'template', panel:'list'};
                break;
            case "sysadmin":
                myobj = {workbench:'admin', bench:'ofile', panel:'list'};
                break;
        }

        if(myobj.workbench==undefined){
            return null;
        }

        var temp = this.getAccessPoint(myobj.workbench, myobj.bench, myobj.panel, 'menu');
        if(temp==null){
            return null;
        }
        myobj.module = temp.module;
        myobj.title = temp.name;
        return myobj;
    }
    hasModuleRights(module){
        var myarr = this.userRights.modules;
        if(myarr && myarr.indexOf(module)>=0){
            return true;
        }
        return false;
    }
    getAccessPoint(workbench, bench, panel, source){
        var myarr = this.userRights.acps;
        if(myarr && myarr.length>0){
            var path = "app://" + workbench  + ":" + bench;
            if(panel && panel.length>0){
                path += ":" + panel;
            }
            for(var i=0; i<myarr.length; i++){
                var temp = myarr[i];
                if(path==temp.path){
                    if(source==undefined || temp.type==source){
                        return temp;
                    }
                }
            }
        }
        return null;
    }
}

var userService = null;
export function provideUserService(opts){
    if(userService==null){
        var temp = new UserService();
        if(!temp.init(opts)){
            return null;
        }
        userService = temp;
    }
    return userService;
}